/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    /* 
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */
	
	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");
	
	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });
    
    //Function for FLK1 - Reg to pole and pole to reg
	gc.databind.registry.bind('widget.ti_widget_spinner19.value', "G_FILK1_REG.FILK1", 
    	function (value) {
    	return Math.round((value/Math.pow(2,16))*200000/(3.14*(2-value/Math.pow(2,16))));	
        }, 
        function (value) {
    	return Math.round((131072*3.14*value)/(3.14*value+200000));
        }
    );

    //Function for FLK2 - Reg to zero and Zero to reg
    gc.databind.registry.bind('widget.ti_widget_spinner18.value', "G_FILK2_REG.FILK2", 
    	function (value) {
	    return Math.round((value/Math.pow(2,19))*200000/(3.14*(2-value/Math.pow(2,19))));	
        },
        function (value) {
	    return Math.round((1048576*3.14*value)/(3.14*value+200000));
        }
    );

    //Function for COMPK1 - Reg to pole and pole to reg
    gc.databind.registry.bind('widget.ti_widget_spinner23.value', "G_COMPK1_REG.COMPK1", 
    	function (value) {
    	return Math.round((value/Math.pow(2,16))*200000/(3.14*(2-value/Math.pow(2,16))));	
        }, 
        function (value) {
    	return Math.round((131072*3.14*value)/(3.14*value+200000));
        }
    );

    //Function for COMPK2 - Reg to zero and Zero to reg
    gc.databind.registry.bind('widget.ti_widget_spinner22.value', "G_COMPK2_REG.COMPK2", 
    	function (value) {
	    return Math.round((value/Math.pow(2,19))*200000/(3.14*(2-value/Math.pow(2,19))));	
        },
        function (value) {
	    return Math.round((1048576*3.14*value)/(3.14*value+200000));
        }
    );
	
	// Event 1 to n bindings
	/* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() 
{
    /* 
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */
	
	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });
	
	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});

/*
*  Boilerplate code for working with components in the application gist
*/
/*
document.addEventListener('polymer-ready', function() {
	var templateObj = document.querySelector('#templateObj');
	templateObj.ready = function(){
	   templateObj.async(function(){
	       // to access a widget with an id of 'widget_id', use templateObj.$.widgetId 
	   },1);
	}
});
*/
